@@ -18,7 +18,7 @@
 // The low should return the lowest point of the interval and
 // the high should return the highest point of the interval.  
 
-template<typename T, typename N=long>
+template<typename T, typename N=long long>
 class IntervalTree {
 public:
   enum color_t {BLACK, RED};
@@ -63,10 +63,17 @@ public:
   IntervalTree();
   ~IntervalTree();
   std::string str() const;
-  T remove(IntervalTree<T,N>::Node *);
+  void remove(N, N, std::vector<T>&);
+  template <class F> void remove(N, N, const F&, std::vector<T>&);
+  void remove_window(N, N, std::vector<T>&);
+  template <class F> void remove_window(N, N, const F&, std::vector<T>&);
   Node * insert(const T&, N, N);
-  std::vector<T> fetch(N, N);
+  void fetch(N, N, std::vector<T>&);
+  void fetch_window(N, N, std::vector<T>&);
 protected:
+  void fetch_node(N, N, std::vector<Node*>&);
+  void fetch_window_node(N, N, std::vector<Node*>&);
+  T remove(Node *);
   Node * GetPredecessorOf(Node *) const;
   Node * GetSuccessorOf(Node *) const;
   void check() const;
@@ -81,6 +88,7 @@ protected:
   Node * nil;
 
   N Overlap(N a1, N a2, N b1, N b2);
+  N Contain(N a1, N a2, N b1, N b2);
   void LeftRotate(Node *);
   void RightRotate(Node *);
   void TreeInsertHelp(Node *);
@@ -104,7 +112,6 @@ private:
 // #define CHECK_INTERVAL_TREE_ASSUMPTIONS 1
 
 template<typename T, typename N> IntervalTree<T,N>::Node::Node() {
-  // std::cerr << "IntervalTree::Node default constructor: " << this << std::endl;
 }
 
 template<typename T, typename N>
@@ -114,19 +121,16 @@ IntervalTree<T,N>::Node::Node(const T& value__, N lowPoint, N highPoint)
     high_(highPoint), 
     maxHigh(highPoint) 
 {
-  // std::cerr << "IntervalTree::Node value constructor: " << this << std::endl;
 }
 
 template<typename T, typename N>
 IntervalTree<T,N>::Node::~Node()
 {
-  // std::cerr << "IntervalTree::Node destructor: " << this << std::endl;
 }
 
 template<typename T, typename N>
 IntervalTree<T,N>::IntervalTree()
 {
-  // std::cerr << "constructor: this=" << this << std::endl;
   nil = new IntervalTree<T,N>::Node();
   nil->left = nil->right = nil->parent = nil;
   nil->color = BLACK;
@@ -541,7 +545,6 @@ void IntervalTree<T,N>::TreePrintHelper(IntervalTree<T,N>::Node* x, std::strings
 
 template<typename T, typename N>
 IntervalTree<T,N>::~IntervalTree() {
-  // std::cerr << "destructor: this=" << this << std::endl;
 
   IntervalTree<T,N>::Node * x = root->left;
   std::vector<IntervalTree<T,N>::Node *> stuffToFree;
@@ -755,6 +758,65 @@ T IntervalTree<T,N>::remove(IntervalTree<T,N>::Node * z){
   return returnValue;
 }
 
+template <typename T, typename N>
+void IntervalTree<T,N>::remove(N low, N high, std::vector<T> &removed) 
+{
+  std::vector<IntervalTree<T,N>::Node*> got;
+  fetch_node(low, high, got);
+  for (typename std::vector<IntervalTree<T,N>::Node*>::const_iterator 
+      i=got.begin(); i!=got.end(); ++i)
+  {
+    removed.push_back((*i)->value());
+    remove(*i);
+  }
+}
+
+template <typename T, typename N> template <typename F> 
+void IntervalTree<T,N>::remove(N low, N high, const F &removeFunctor, std::vector<T> &removed) 
+{
+  std::vector<IntervalTree<T,N>::Node*> got;
+  fetch_node(low, high, got);
+  for (typename std::vector<IntervalTree<T,N>::Node*>::const_iterator 
+      i=got.begin(); i!=got.end(); ++i)
+  {
+    if (removeFunctor((*i)->value(), (*i)->low(), (*i)->high())) {
+      removed.push_back((*i)->value());
+      remove(*i);
+    }
+  }
+}
+
+template <typename T, typename N>
+void IntervalTree<T,N>::remove_window(N low, N high, std::vector<T> &removed) 
+{
+  std::vector<IntervalTree<T,N>::Node*> got;
+  fetch_window_node(low, high, got);
+  for (typename std::vector<IntervalTree<T,N>::Node*>::const_iterator 
+      i=got.begin(); i!=got.end(); ++i)
+  {
+    removed.push_back((*i)->value());
+    remove(*i);
+  }
+}
+
+template <typename T, typename N> template <typename F>
+void IntervalTree<T,N>::remove_window(
+    N low, 
+    N high, 
+    const F& removeFunctor, 
+    std::vector<T> &removed) 
+{
+  std::vector<IntervalTree<T,N>::Node*> got;
+  fetch_window_node(low, high, got);
+  for (typename std::vector<IntervalTree<T,N>::Node*>::const_iterator 
+      i=got.begin(); i!=got.end(); ++i)
+  {
+    if (removeFunctor((*i)->value(), (*i)->low(), (*i)->high())) {
+      removed.push_back((*i)->value());
+      remove(*i);
+    }
+  }
+}
 
 /***********************************************************************/
 /*  FUNCTION:  Overlap */
@@ -775,6 +837,11 @@ N IntervalTree<T,N>::Overlap(N a1, N a2, N b1, N b2) {
 }
 
 
+template<typename T, typename N>
+N IntervalTree<T,N>::Contain(N a1, N a2, N b1, N b2) {
+  return a1 <= b1 && b2 <= a2;
+}
+
 /***********************************************************************/
 /*  FUNCTION:  fetch */
 /**/
@@ -814,8 +881,7 @@ N IntervalTree<T,N>::Overlap(N a1, N a2, N b1, N b2) {
 /*  of the left child of root as well as the right child of root. */
 
 template<typename T, typename N>
-std::vector<T> IntervalTree<T,N>::fetch(N low, N high)  {
-  std::vector<T> enumResultStack;
+void IntervalTree<T,N>::fetch(N low, N high, std::vector<T> &enumResultStack)  {
   IntervalTree<T,N>::Node* x=root->left;
   bool stuffToDo = (x != nil);
   
@@ -859,10 +925,157 @@ std::vector<T> IntervalTree<T,N>::fetch(N low, N high)  {
   assert((recursionNodeStack.size() == 1)
          || !"recursionStack not empty when exiting IntervalTree::fetch");
 #endif
-  return(enumResultStack);   
+}
+
+template<typename T, typename N>
+void IntervalTree<T,N>::fetch_node(
+    N low, 
+    N high, 
+    std::vector<typename IntervalTree<T,N>::Node*> &enumResultStack)  
+{
+  IntervalTree<T,N>::Node* x=root->left;
+  bool stuffToDo = (x != nil);
+  
+  // Possible speed up: add min field to prune right searches //
+
+#ifdef DEBUG_ASSERT
+  assert((recursionNodeStack.size() == 1)
+         || !"recursionStack not empty when entering IntervalTree::fetch");
+#endif
+  currentParent = 0;
+
+  while(stuffToDo) {
+    if (Overlap(low,high,x->key,x->high_) ) {
+      enumResultStack.push_back(x);
+      recursionNodeStack[currentParent].tryRightBranch=true;
+    }
+    if(x->left->maxHigh >= low) { // implies x != nil 
+      recursionNodeStack.push_back(IntervalTree<T,N>::it_recursion_node());
+      recursionNodeStack.back().start_node = x;
+      recursionNodeStack.back().tryRightBranch = false;
+      recursionNodeStack.back().parentIndex = currentParent;
+      currentParent = recursionNodeStack.size()-1;
+      x = x->left;
+    } else {
+      x = x->right;
+    }
+    stuffToDo = (x != nil);
+    while( (!stuffToDo) && (recursionNodeStack.size() > 1) ) {
+        IntervalTree<T,N>::it_recursion_node back = recursionNodeStack.back();
+        recursionNodeStack.pop_back();
+
+        if(back.tryRightBranch) {
+          x=back.start_node->right;
+          currentParent=back.parentIndex;
+          recursionNodeStack[currentParent].tryRightBranch=true;
+          stuffToDo = ( x != nil);
+        }
+    }
+  }
+#ifdef DEBUG_ASSERT
+  assert((recursionNodeStack.size() == 1)
+         || !"recursionStack not empty when exiting IntervalTree::fetch");
+#endif
 }
         
+template<typename T, typename N>
+void IntervalTree<T,N>::fetch_window(N low, N high, std::vector<T> &enumResultStack)  
+{
+  IntervalTree<T,N>::Node* x=root->left;
+  bool stuffToDo = (x != nil);
+  
+  // Possible speed up: add min field to prune right searches //
 
+#ifdef DEBUG_ASSERT
+  assert((recursionNodeStack.size() == 1)
+         || !"recursionStack not empty when entering IntervalTree::fetch_window");
+#endif
+  currentParent = 0;
+
+  while(stuffToDo) {
+    if (Contain(low,high,x->key,x->high_) ) {
+      enumResultStack.push_back(x->value());
+      recursionNodeStack[currentParent].tryRightBranch=true;
+    }
+    if(x->left->maxHigh >= low) { // implies x != nil 
+      recursionNodeStack.push_back(IntervalTree<T,N>::it_recursion_node());
+      recursionNodeStack.back().start_node = x;
+      recursionNodeStack.back().tryRightBranch = false;
+      recursionNodeStack.back().parentIndex = currentParent;
+      currentParent = recursionNodeStack.size()-1;
+      x = x->left;
+    } else {
+      x = x->right;
+    }
+    stuffToDo = (x != nil);
+    while( (!stuffToDo) && (recursionNodeStack.size() > 1) ) {
+        IntervalTree<T,N>::it_recursion_node back = recursionNodeStack.back();
+        recursionNodeStack.pop_back();
+
+        if(back.tryRightBranch) {
+          x=back.start_node->right;
+          currentParent=back.parentIndex;
+          recursionNodeStack[currentParent].tryRightBranch=true;
+          stuffToDo = ( x != nil);
+        }
+    }
+  }
+#ifdef DEBUG_ASSERT
+  assert((recursionNodeStack.size() == 1)
+         || !"recursionStack not empty when exiting IntervalTree::fetch");
+#endif
+}
+
+template<typename T, typename N>
+void IntervalTree<T,N>::fetch_window_node(
+     N low, 
+     N high, 
+     std::vector<typename IntervalTree<T,N>::Node*> &enumResultStack)  
+{
+  IntervalTree<T,N>::Node* x=root->left;
+  bool stuffToDo = (x != nil);
+  
+  // Possible speed up: add min field to prune right searches //
+
+#ifdef DEBUG_ASSERT
+  assert((recursionNodeStack.size() == 1)
+         || !"recursionStack not empty when entering IntervalTree::fetch_window_node");
+#endif
+  currentParent = 0;
+
+  while(stuffToDo) {
+    if (Contain(low,high,x->key,x->high_) ) {
+      enumResultStack.push_back(x);
+      recursionNodeStack[currentParent].tryRightBranch=true;
+    }
+    if(x->left->maxHigh >= low) { // implies x != nil 
+      recursionNodeStack.push_back(IntervalTree<T,N>::it_recursion_node());
+      recursionNodeStack.back().start_node = x;
+      recursionNodeStack.back().tryRightBranch = false;
+      recursionNodeStack.back().parentIndex = currentParent;
+      currentParent = recursionNodeStack.size()-1;
+      x = x->left;
+    } else {
+      x = x->right;
+    }
+    stuffToDo = (x != nil);
+    while( (!stuffToDo) && (recursionNodeStack.size() > 1) ) {
+        IntervalTree<T,N>::it_recursion_node back = recursionNodeStack.back();
+        recursionNodeStack.pop_back();
+
+        if(back.tryRightBranch) {
+          x=back.start_node->right;
+          currentParent=back.parentIndex;
+          recursionNodeStack[currentParent].tryRightBranch=true;
+          stuffToDo = ( x != nil);
+        }
+    }
+  }
+#ifdef DEBUG_ASSERT
+  assert((recursionNodeStack.size() == 1)
+         || !"recursionStack not empty when exiting IntervalTree::fetch");
+#endif
+}
 
 template<typename T, typename N>
 bool IntervalTree<T,N>::CheckMaxHighFieldsHelper(IntervalTree<T,N>::Node * y, 
